'use strict'

exports.__esModule = true
exports.useAsync = useAsync
exports.useAsyncEffect = useAsyncEffect

var _react = _interopRequireDefault(require('react'))

var _useMemoOne = require('use-memo-one')

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj}
}

const {useReducer, useEffect} = _react.default

function _ref(prev, action) {
  return {
    // This is the current status of the promise or async/await function. A
    // promise or async/await can only be in one state at a time.
    status: action.status,
    // The value is persisted between 'success' statuses. This means I can
    // still display things that depend on my current value while my new
    // value is loading.
    value: action.status === 'success' ? action.value : prev.value,
    // Errors get reset each time we leave the error state. There's really
    // no use in keeping those around. They go stale once we leave.
    error: action.status === 'error' ? action.error : void 0,
    // The id is used for invalidating callbacks when they're cancelled and
    // preventing race conditions by keeping the cancelled state local to the
    // individual callback.
    id: action.status === 'cancelled' ? ++prev.id : prev.id,
  }
}

function useAsync(asyncCallback, dependencies = []) {
  const [state, dispatch] = useReducer(_ref, {
    status: 'idle',
    value: void 0,
    error: void 0,
    id: 0,
  }) // Creates a stable callback that manages our loading/success/error status updates
  // as the callback is invoked. This callback will not dispatch state once its been
  // invalidated.

  const callback = (0, _useMemoOne.useCallbackOne)(
    Object.assign(
      async (...args) => {
        dispatch({
          status: 'loading',
        })

        try {
          const value = await asyncCallback(...args)
          !callback.cancelled &&
            dispatch({
              status: 'success',
              value,
            })
        } catch (error) {
          !callback.cancelled &&
            dispatch({
              status: 'error',
              error,
            })
        }
      },
      {
        cancelled: false,
      }
    ), // Makes sure changing the id (happens when cancel() is invoked) always invalidates
    // the current callback
    // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies.concat(state.id)
  ) // Cancels any pending async stuff when the callback is invalidated or the hook
  // is unmounted

  function _ref2() {
    callback.cancelled = true
  }

  useEffect(() => _ref2, [callback])

  function _ref3() {
    return dispatch({
      status: 'cancelled',
    })
  }

  return [
    (0, _useMemoOne.useMemoOne)(
      () => ({
        status: state.status,
        value: state.value,
        error: state.error,
        cancel: _ref3,
      }),
      [state]
    ),
    callback,
  ]
}

function useAsyncEffect(asyncCallback, dependencies) {
  const [state, callback] = useAsync(
    asyncCallback,
    dependencies || [Math.random()]
  ) // Runs the callback each time the dependencies change and cancels any
  // pending callbacks that were running previously

  useEffect(
    () => {
      callback()
    }, // eslint-disable-next-line react-hooks/exhaustive-deps
    dependencies
  )
  return state
}
